from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class FixUpdatesMigration(MigrationABC):
    name = "1.1.7_FixUpdatesMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_ServerHistory
                    ADD DefaultRoleId BIGINT NULL AFTER LoginMessageChannelId;
                """
            )
        )

        self._cursor.execute(
            str(
                """ALTER TABLE CFG_TechnicianHistory ADD FeatureFlags JSON NULL DEFAULT ('{}') AFTER CacheMaxMessages;"""
            )
        )

        self._cursor.execute(
            str(
                """ALTER TABLE CFG_ServerHistory ADD FeatureFlags JSON NULL DEFAULT ('{}') AFTER LoginMessageChannelId;"""
            )
        )

        self._exec(__file__, "config/server.sql")
        self._exec(__file__, "config/technician.sql")

    def downgrade(self):
        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_ServerHistory DROP COLUMN DefaultRoleId;
                """
            )
        )
        self._cursor.execute("ALTER TABLE CFG_TechnicianHistory DROP COLUMN FeatureFlags;")
        self._cursor.execute("ALTER TABLE CFG_ServerHistory DROP COLUMN FeatureFlags;")
