import json
from datetime import datetime

from cpl_core.configuration import ConfigurationModelABC
from cpl_core.database import TableABC
from cpl_query.extension import List

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum


class TechnicianConfig(TableABC, ConfigurationModelABC):
    def __init__(
        self,
        help_command_reference_url: str,
        wait_for_restart: int,
        wait_for_shutdown: int,
        cache_max_messages: int,
        feature_flags: dict[FeatureFlagsEnum],
        technician_ids: List[int],
        ping_urls: List[str],
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._id = id
        self._help_command_reference_url = help_command_reference_url
        self._wait_for_restart = wait_for_restart
        self._wait_for_shutdown = wait_for_shutdown
        self._cache_max_messages = cache_max_messages
        self._feature_flags = feature_flags
        self._technician_ids = technician_ids
        self._ping_urls = ping_urls

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._id

    @property
    def help_command_reference_url(self) -> str:
        return self._help_command_reference_url

    @help_command_reference_url.setter
    def help_command_reference_url(self, value: str):
        self._help_command_reference_url = value

    @property
    def wait_for_restart(self) -> int:
        return self._wait_for_restart

    @wait_for_restart.setter
    def wait_for_restart(self, value: int):
        self._wait_for_restart = value

    @property
    def wait_for_shutdown(self) -> int:
        return self._wait_for_shutdown

    @wait_for_shutdown.setter
    def wait_for_shutdown(self, value: int):
        self._wait_for_shutdown = value

    @property
    def cache_max_messages(self) -> int:
        return self._cache_max_messages

    @cache_max_messages.setter
    def cache_max_messages(self, value: int):
        self._cache_max_messages = value

    @property
    def feature_flags(self) -> dict[FeatureFlagsEnum]:
        return self._feature_flags

    @feature_flags.setter
    def feature_flags(self, value: dict[FeatureFlagsEnum]):
        self._feature_flags = value

    @property
    def technician_ids(self) -> List[int]:
        return self._technician_ids

    @technician_ids.setter
    def technician_ids(self, value: List[int]):
        self._technician_ids = value

    @property
    def ping_urls(self) -> List[str]:
        return self._ping_urls

    @ping_urls.setter
    def ping_urls(self, value: List[str]):
        self._ping_urls = value

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
                SELECT * FROM `CFG_Technician`;
            """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
                SELECT * FROM `CFG_Technician`
                WHERE `Id` = {id};
            """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
                INSERT INTO `CFG_Technician` (
                    `HelpCommandReferenceUrl`, `WaitForRestart`, `WaitForShutdown`, `CacheMaxMessages`, `FeatureFlags`
                ) VALUES (
                    '{self._help_command_reference_url}',
                    {self._wait_for_restart},
                    {self._wait_for_shutdown},
                    {self._cache_max_messages},
                    '{json.dumps(self._feature_flags)}'
                );
            """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
                UPDATE `CFG_Technician`
                SET `HelpCommandReferenceUrl` = '{self._help_command_reference_url}',
                `WaitForRestart` = {self._wait_for_restart},
                `WaitForShutdown` = {self._wait_for_shutdown},
                `CacheMaxMessages` = {self._cache_max_messages},
                `FeatureFlags` = '{json.dumps(self._feature_flags)}'
                WHERE `Id` = {self._id};
            """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
                DELETE FROM `CFG_Technician`
                WHERE `Id` = {self._id};
            """
        )
