from typing import List

from bot_data.abc.achievement_repository_abc import AchievementRepositoryABC
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.game_server_repository_abc import GameServerRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_game_server_repository_abc import UserJoinedGameServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user import User


class AchievementAttributeResolver:
    def __init__(
        self,
        auto_roles: AutoRoleRepositoryABC,
        clients: ClientRepositoryABC,
        known_users: KnownUserRepositoryABC,
        levels: LevelRepositoryABC,
        servers: ServerRepositoryABC,
        game_servers: GameServerRepositoryABC,
        user_joined_servers: UserJoinedServerRepositoryABC,
        user_joined_voice_channels: UserJoinedVoiceChannelRepositoryABC,
        user_joined_game_server: UserJoinedGameServerRepositoryABC,
        users: UserRepositoryABC,
        achievements: AchievementRepositoryABC,
    ):
        self._auto_roles = auto_roles
        self._clients = clients
        self._known_users = known_users
        self._levels = levels
        self._servers = servers
        self._game_servers = game_servers
        self._user_joined_servers = user_joined_servers
        self._user_joined_voice_channels = user_joined_voice_channels
        self._user_joined_game_server = user_joined_game_server
        self._users = users
        self._achievements = achievements

    def get_played_on_game_server(self, user: User) -> List[str]:
        joins = self._user_joined_game_server.get_user_joined_game_servers_by_user_id(user.id)
        return joins.select(lambda x: x.game_server.name)

    def get_last_ontime_hours(self, user: User) -> int:
        ujvs = self._user_joined_voice_channels.get_user_joined_voice_channels_by_user_id(user.id)
        return int(str(ujvs.max(lambda join: (join.leaved_on - join.joined_on).total_seconds() / 3600)))
