from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.logging import LoggerABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.helper.command_checks import CommandChecks
from modules.base.forms.bug_report_form import BugReportForm
from modules.base.forms.complaint_form import ComplaintForm


class SubmitGroup(DiscordCommandABC):
    def __init__(self, services: ServiceProviderABC, logger: LoggerABC, bot: DiscordBotServiceABC):
        DiscordCommandABC.__init__(self)

        self._services = services
        self._logger = logger
        self._bot = bot

    @commands.hybrid_group()
    @commands.guild_only()
    async def submit(self, ctx: Context):
        pass

    @submit.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    async def complaint(self, ctx: Context):
        self._logger.debug(__name__, f"Received command complaint {ctx}")
        complaint_form: ComplaintForm = self._services.get_service(ComplaintForm)
        await ctx.interaction.response.send_modal(complaint_form)
        self._logger.trace(__name__, f"Finished command complaint {ctx}")

    @submit.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    async def bug_report(self, ctx: Context):
        self._logger.debug(__name__, f"Received command complaint {ctx}")
        bug_form: BugReportForm = self._services.get_service(BugReportForm)
        await ctx.interaction.response.send_modal(bug_form)
        self._logger.trace(__name__, f"Finished command complaint {ctx}")
